//-----------------------------------------------------------------------------
// File: SMAP.fx   
//
// Similar to Microsoft ShadowMap.fx  ( fennec original )
//
// Removed g_fCosTheta
//-----------------------------------------------------------------------------

#define SMAP_SIZE 1024

texture  ShadowMap;

float4x4 WorldViewProj;
float4x4 TexTransform;

float Bias = 0.0f;
float FilterGain = 1.00f;

float VSM_Zstart = 0.0f;
float VSM_Zrange = 1.0f;


// -------------------------------------------- if using register s0 ShadowMapSampler produces only garbage
sampler ShadowMapSampler : register(s7)
= sampler_state
{
    Texture = <ShadowMap>;
    MinFilter = Point;
    MagFilter = Point;   
    AddressU = Border;
    AddressV = Border;
    BorderColor = float4(1.0,1.0,1.0,1.0);
};






// ****************************************************************************
//   GENERATE SHADOW MAP
// ****************************************************************************

//-----------------------------------------------------------------------------
// Vertex Shader: VertShadow
//-----------------------------------------------------------------------------
void VertShadow(	float4 Pos		: POSITION,
					float3 Normal	: NORMAL,
				out float4 oPos		: POSITION,
				out float2 oDepth	: TEXCOORD0 )
{
    // ------------------------------------------------------ compute the projected coordinate
    oPos = mul(Pos, WorldViewProj);

    // ------------------------------------------------------ store z and w in texcoord0
    oDepth.xy = oPos.zw;
}

//-----------------------------------------------------------------------------
// Pixel Shader: PixShadow
//-----------------------------------------------------------------------------
void PixShadow(		float2 iDepth	: TEXCOORD0,
                out float4 oColor	: COLOR )
{
    // ------------------------------------------------------- depth is z / w
    oColor = iDepth.x / iDepth.y;    
}

//-----------------------------------------------------------------------------
// Pixel Shader: PixShadow_1_1  ( version for PS 1.1 ) 
//-----------------------------------------------------------------------------
void PixShadow_1_1(out float4 oColor  : COLOR )
{
    oColor = float4(0,0,0,1);
}


//-----------------------------------------------------------------------------
// Technique: RenderShadow
// Desc: Renders the shadow map
//-----------------------------------------------------------------------------
technique RenderShadow
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertShadow();
        PixelShader = compile ps_2_0 PixShadow();
        //PixelShader = compile ps_1_0 PixShadow_1_1();
        
        //ZEnable			= True;
        //AlphaBlendEnable	= False;
        //Lighting			= False;
        //CullMode			= CCW;
        //ColorWriteEnable	= 0;
    }
}






// ****************************************************************************
//   USE SHADOW MAP - TEST
// ****************************************************************************

//-----------------------------------------------------------------------------
// Vertex Shader: VertScene_Test
//-----------------------------------------------------------------------------
void VertScene_Test(float4 iPos				: POSITION,
					out float4 oPos			: POSITION, 
					out float4 oPosLight	: TEXCOORD0)
{    
    // -------------------------------------------------- transform position to screen coord
	oPos = mul(iPos, WorldViewProj);
	
	// -------------------------------------------------- transform position to light projection space
    //													  as if the camera is looking out from the light
	oPosLight = mul(iPos, TexTransform);
}


//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Test
//-----------------------------------------------------------------------------
float4 PixScene_Test(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
	// - - 6 - - 
 	// - 1 - 2 -
    // 5 - - - 7
    // - 4 - 3 -
    // - - 8 - -   percentage closer filtering ( PCF )( 5x5-8 )  ---  (8 texture, 24 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 2.0 / SMAP_SIZE;
    float delta2 = 4.0 / SMAP_SIZE;
       
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, 0));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(0, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, 0));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/8);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}










// ****************************************************************************
//   USE SHADOW MAP
// ****************************************************************************

//-----------------------------------------------------------------------------
// Vertex Shader: VertScene
//-----------------------------------------------------------------------------
void VertScene( float4 iPos				: POSITION,
                out float4 oPos			: POSITION, 
				out float4 oPosLight	: TEXCOORD0)
{    
    // -------------------------------------------------- transform position to screen coord
	oPos = mul(iPos, WorldViewProj);
	
	// -------------------------------------------------- transform position to light projection space
    //													  as if the camera is looking out from the light
	oPosLight = mul(iPos, TexTransform);
}




//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Fast5
//-----------------------------------------------------------------------------
float4 PixScene_Fast5(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
	// - - 6 - - 
 	// - 1 - 2 -
    // 5 - - - 7
    // - 4 - 3 -
    // - - 8 - -   percentage closer filtering ( PCF )( 5x5-8 )  ---  (8 texture, 24 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 1.5 / SMAP_SIZE;
    float delta2 = 3.0 / SMAP_SIZE;
       
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, 0));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(0, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, 0));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/8);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}





//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Fast4
//-----------------------------------------------------------------------------
float4 PixScene_Fast4(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
	// - - 6 - - 
 	// - 1 - 2 -
    // 5 - - - 7
    // - 4 - 3 -
    // - - 8 - -   percentage closer filtering ( PCF )( 5x5-8 )  ---  (8 texture, 24 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 1.0 / SMAP_SIZE;
    float delta2 = 2.0 / SMAP_SIZE;
       
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, 0));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(0, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, 0));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/8);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}




//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Fast3
//-----------------------------------------------------------------------------
float4 PixScene_Fast3(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
	// - - 6 - - 
 	// - 1 - 2 -
    // 5 - - - 7
    // - 4 - 3 -
    // - - 8 - -   percentage closer filtering ( PCF )( 5x5-8 )  ---  (8 texture, 24 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 0.5 / SMAP_SIZE;
    float delta2 = 1.0 / SMAP_SIZE;
       
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, 0));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(0, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, 0));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/8);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}




//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Fast2
//-----------------------------------------------------------------------------
float4 PixScene_Fast2(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
 	// 1 - 2 
    // - - - 
    // 4 - 3   percentage closer filtering ( PCF )( 3x3 ) ( large 2 ) (4 texture, 14 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 1.5 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/4);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}



//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Fast1
//-----------------------------------------------------------------------------
float4 PixScene_Fast1(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


	// -----------------------------------------------------------------------------------------
 	// 1 - 2 
    // - - - 
    // 4 - 3   percentage closer filtering ( PCF )( 3x3 ) ( large 2 ) (4 texture, 14 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 1.0 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/4);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}




//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Hard
//-----------------------------------------------------------------------------
float4 PixScene_Hard(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


    // ======================================================================================================
    //  find if pixel is in shadow		-   (1 texture, 7 arithmetic)
    // ======================================================================================================  
    float v0;
    v0 = tex2D(ShadowMapSampler, TexCoord) >= TestDepth; 
	
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v0 = v0 * FilterGain;
	return float4(v0, v0, v0, v0);
}


//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft1
//-----------------------------------------------------------------------------
float4 PixScene_Soft1( float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


    // ======================================================================================================
    //  Find if pixel is in shadow using percentage closest filtering
    // ======================================================================================================  

    // -------------------------------------------------------------------------------------
	// 3 - 4
    // - 0 -
    // 1 - 2   percentage closer filtering ( PCF )( 3x3-5a )  -  (5 texture, 24 arithmetic)
	// -------------------------------------------------------------------------------------
	float v0;
    float delta = 0.5 / SMAP_SIZE;
    v0  = (tex2D( ShadowMapSampler, TexCoord )							>= TestDepth); 
    v0 += (tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta))	>= TestDepth);
    v0 += (tex2D( ShadowMapSampler, TexCoord + float2(delta, delta))	>= TestDepth);
    v0 += (tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta))	>= TestDepth);
    v0 += (tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta))	>= TestDepth);
    v0 /= 5.0f;
	
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v0 = v0 * FilterGain;
	return float4(v0, v0, v0, v0);
}


//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft2
//-----------------------------------------------------------------------------
float4 PixScene_Soft2( float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;


    // ======================================================================================================
    //  Find if pixel is in shadow using percentage closest filtering
    // ======================================================================================================  
		
	// -----------------------------------------------------------------------------------------
	// - - 6 - - 
 	// - 1 - 2 -
    // 5 - - - 7
    // - 4 - 3 -
    // - - 8 - -   percentage closer filtering ( PCF )( 5x5-8 )  ---  (8 texture, 24 arithmetic)
    // -----------------------------------------------------------------------------------------
    float4 v0;
    float4 v1;
    float delta = 0.5 / SMAP_SIZE;
    float delta2 = 1.0 / SMAP_SIZE;
       
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, 0));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(0, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, 0));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/8);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}




//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft3
//-----------------------------------------------------------------------------
float4 PixScene_Soft3(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // --------------------------------------- nearest 12		(12 texture, 34 arithmetic)

    float4 v0;
    float4 v1;
    float delta = 0.5 / SMAP_SIZE;
    float delta2 = 1.0 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/12);
    
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}



//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft4
//-----------------------------------------------------------------------------
float4 PixScene_Soft4(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-16 )	(16 texture, 44 arithmetic)
    // ----------------------------------------------------------------------------
    // -- -- -- 13 -- -- --
	// -- -- 06 -- 07 -- --
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // -- -- 11 -- 10 -- --
    // -- -- -- 15 -- -- --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 1.0 / 3 / SMAP_SIZE;
    float delta2 = 2.0 / 3 / SMAP_SIZE;
    float delta3 = 3.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/16);
    
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}



//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft5
//-----------------------------------------------------------------------------
float4 PixScene_Soft5(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-24 )	(24 texture, 64 arithmetic)
    // ----------------------------------------------------------------------------
    // -- 18 -- 13 -- 19 --
	// 17 -- 06 -- 07 -- 20
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // 24 -- 11 -- 10 -- 21
    // -- 23 -- 15 -- 22 --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 1.0 / 3 / SMAP_SIZE;
    float delta2 = 2.0 / 3 / SMAP_SIZE;
    float delta3 = 3.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta3, delta2));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta3, -delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/24);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}





//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft6  ( 24 tap - large 2 )
//-----------------------------------------------------------------------------
float4 PixScene_Soft6(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-24 )	(24 texture, 64 arithmetic)
    // ----------------------------------------------------------------------------
    // -- 18 -- 13 -- 19 --
	// 17 -- 06 -- 07 -- 20
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // 24 -- 11 -- 10 -- 21
    // -- 23 -- 15 -- 22 --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 2.0 / 3 / SMAP_SIZE;
    float delta2 = 4.0 / 3 / SMAP_SIZE;
    float delta3 = 6.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta3, delta2));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta3, -delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/24);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}



//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft7     ( 24 tap - large 3 )
//-----------------------------------------------------------------------------
float4 PixScene_Soft7(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-24 )	(24 texture, 64 arithmetic)
    // ----------------------------------------------------------------------------
    // -- 18 -- 13 -- 19 --
	// 17 -- 06 -- 07 -- 20
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // 24 -- 11 -- 10 -- 21
    // -- 23 -- 15 -- 22 --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 3.0 / 3 / SMAP_SIZE;
    float delta2 = 6.0 / 3 / SMAP_SIZE;
    float delta3 = 9.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta3, delta2));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta3, -delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/24);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}



//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft8     ( 24 tap - large 4 )
//-----------------------------------------------------------------------------
float4 PixScene_Soft8(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-24 )	(24 texture, 64 arithmetic)
    // ----------------------------------------------------------------------------
    // -- 18 -- 13 -- 19 --
	// 17 -- 06 -- 07 -- 20
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // 24 -- 11 -- 10 -- 21
    // -- 23 -- 15 -- 22 --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 4.0 / 3 / SMAP_SIZE;
    float delta2 = 8.0 / 3 / SMAP_SIZE;
    float delta3 = 12.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta3, delta2));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta3, -delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/24);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}


//-----------------------------------------------------------------------------
// Pixel Shader: PixScene_Soft9     ( 24 tap - large 6 )
//-----------------------------------------------------------------------------
float4 PixScene_Soft9(float4 iPosLight : TEXCOORD0) : COLOR
{
	// ======================================================================================================
    // transform from RT space to texture space 
    // ======================================================================================================
    float2 TexCoord = iPosLight.xy / iPosLight.w;
    float TestDepth = (iPosLight.z - Bias) / iPosLight.w;

    // ======================================================================================================
    //  find if pixel is in shadow
    // ======================================================================================================      
    
    // percentage closer filtering ( PCF )( 7x7-24 )	(24 texture, 64 arithmetic)
    // ----------------------------------------------------------------------------
    // -- 18 -- 13 -- 19 --
	// 17 -- 06 -- 07 -- 20
 	// -- 05 -- 04 -- 08 --
    // 16 -- 01 -- 02 -- 14
    // -- 12 -- 03 -- 09 --
    // 24 -- 11 -- 10 -- 21
    // -- 23 -- 15 -- 22 --  
    // ----------------------------------------------------------------------------

    float4 v0;
    float4 v1;
    float delta = 6.0 / 3 / SMAP_SIZE;
    float delta2 = 12.0 / 3 / SMAP_SIZE;
    float delta3 = 18.0 / 3 / SMAP_SIZE;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta));
    v1 = v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta, delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta, delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta, -delta2));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta, -delta2));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, 0));  
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta3, 0));  
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(0, -delta3)); 
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(0, delta3));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(delta2, delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(delta3, delta2));
    v1 += v0 >= TestDepth;
    
    v0.r = tex2D( ShadowMapSampler, TexCoord + float2(delta3, -delta2));
    v0.g = tex2D( ShadowMapSampler, TexCoord + float2(delta2, -delta3));
    v0.b = tex2D( ShadowMapSampler, TexCoord + float2(-delta2, -delta3));
    v0.a = tex2D( ShadowMapSampler, TexCoord + float2(-delta3, -delta2));
    v1 += v0 >= TestDepth;
    
    v1 = dot(v1, 1.0/24);
    
	
	// ======================================================================================================
	//  create the return value
	// ======================================================================================================
	v1 = v1 * FilterGain;
	return v1;
}





//-----------------------------------------------------------------------------
// Technique: RenderScene
//-----------------------------------------------------------------------------
technique RenderScene_Test
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene_Test();
        PixelShader = compile ps_2_0 PixScene_Test();
    }
}


technique RenderScene_Fast5
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Fast5();
    }
}
technique RenderScene_Fast4
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Fast4();
    }
}
technique RenderScene_Fast3
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Fast3();
    }
}
technique RenderScene_Fast2
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Fast2();
    }
}
technique RenderScene_Fast1
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Fast1();
    }
}
technique RenderScene_Hard
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Hard();
    }
}
technique RenderScene_Soft1
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft1();
    }
}
technique RenderScene_Soft2
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft2();
    }
}
technique RenderScene_Soft3
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft3();
    }
}
technique RenderScene_Soft4
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft4();
    }
}
technique RenderScene_Soft5
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft5();
    }
}
technique RenderScene_Soft6
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft6();
    }
}
technique RenderScene_Soft7
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft7();
    }
}
technique RenderScene_Soft8
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft8();
    }
}
technique RenderScene_Soft9
{
    pass p0
    {
        VertexShader = compile vs_1_1 VertScene();
        PixelShader = compile ps_2_0 PixScene_Soft9();
    }
}





// ****************************************************************************
//   DRAW SHADOW MAP
// ****************************************************************************

void DrawShadowQuadVS(float4 iPos		: POSITION,
					  float4 iTex		: TEXCOORD0,
					  out float4 oPos	: POSITION, 
					  out float4 oTex	: TEXCOORD0)
{
	oPos = iPos;
	oTex = iTex;
}


float4 DrawShadowQuadPS(float2 iTex : TEXCOORD0) : COLOR
{
    float c = tex2D(ShadowMapSampler, iTex); 
    c = (c - VSM_Zstart) / VSM_Zrange;
    return float4(c, c, c, 1);
}


technique DrawShadowQuad
{
    Pass P0
    {
        VertexShader = compile vs_1_1 DrawShadowQuadVS();
        PixelShader = compile ps_2_0 DrawShadowQuadPS();
    }
}

